@@ -70,6 +70,7 @@ class GIGABASE_DLL_ENTRY dbSelection {
 	nRows += 1;
     }
 
+    void truncate(size_t from, size_t length);
     void toArray(oid_t* oids);
     void merge(dbSelection& selection);
 
@@ -126,6 +127,7 @@ class GIGABASE_DLL_ENTRY dbL2List {
 class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
     friend class dbDatabase;
     friend class dbHashTable;
+    friend class dbRtreePage;
     friend class dbBtreePage;
     friend class dbBtreeLeafPage;
     friend class dbSubSql;
@@ -137,9 +139,13 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
 
     void remove();
 
-    bool isEmpty() { return currId == 0; }
+    bool isEmpty() { 
+	return currId == 0; 
+    }
 
-    bool isLimitReached() { return selection.nRows >= limit; }
+    bool isLimitReached() { 
+	return selection.nRows >= limit || selection.nRows >= stmtLimitLen; 
+    }
 
     int select(dbQuery& query, dbCursorType aType, void* paramStruct = NULL) {
 	paramBase = paramStruct;
@@ -252,6 +258,10 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
     bool               removed; // current record was removed
     bool               updateInProgress;
 
+    size_t             stmtLimitStart;
+    size_t             stmtLimitLen;
+    size_t             nSkipped;
+
     void checkForDuplicates() {
 	if (!eliminateDuplicates && limit > 1) {
 	    eliminateDuplicates = true;
@@ -275,8 +285,21 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
 	}
     }
 
+    void setStatementLimit(dbQuery const& q) { 
+	stmtLimitStart = q.stmtLimitStartPtr != NULL ? (nat4)*q.stmtLimitStartPtr : q.stmtLimitStart;
+	stmtLimitLen = q.stmtLimitLenPtr != NULL ? (nat4)*q.stmtLimitLenPtr : q.stmtLimitLen;
+    }
+
+    void truncateSelection() { 
+	selection.truncate(stmtLimitStart, stmtLimitLen);
+    }
+
     bool add(oid_t oid) {
-	if (selection.nRows < limit) {
+	if (selection.nRows < limit && selection.nRows < stmtLimitLen) {
+	    if (nSkipped < stmtLimitStart) { 
+		nSkipped += 1;
+		return true;
+	    }
 	    if (eliminateDuplicates) {
 		if (bitmap[oid >> 5] & (1 << (oid & 31))) {
 		    return true;
